@@ -68,7 +68,8 @@ macro_rules! sql_insert {
         let fieldnames = &[ $( { $field } ,)+ ].join(",");
         let fieldvars = &[ $( { format!(":{}", $field) } ,)+ ].join(",");
         let sql = format!("INSERT INTO {} ({}) VALUES ({})", table, fieldnames, fieldvars);
-        $txn.execute(&sql, &[$(
+        let mut stmt = $txn.prepare_cached(&sql)?;
+        stmt.execute(&[$(
             (format!(":{}", $field).as_str(), &$val as &dyn holochain_sqlite::rusqlite::ToSql),
         )+])
     }};
@@ -132,28 +133,26 @@ pub fn insert_element_scratch(
 }
 
 /// Insert a [`DhtOp`] into the database.
-pub fn insert_op(txn: &mut Transaction, op: DhtOpHashed) -> StateMutationResult<()> {
-    let (op, hash) = op.into_inner();
+pub fn insert_op(txn: &mut Transaction, op: &DhtOpHashed) -> StateMutationResult<()> {
+    let hash = op.as_hash();
+    let op = op.as_content();
     let op_light = op.to_light();
     let header = op.header();
     let timestamp = header.timestamp();
     let signature = op.signature().clone();
     if let Some(entry) = op.entry() {
-        let entry_hashed = EntryHashed::with_pre_hashed(
-            entry.clone(),
-            header
-                .entry_hash()
-                .ok_or_else(|| DhtOpError::HeaderWithoutEntry(header.clone()))?
-                .clone(),
-        );
-        insert_entry(txn, entry_hashed)?;
+        let entry_hash = header
+            .entry_hash()
+            .ok_or_else(|| DhtOpError::HeaderWithoutEntry(header.clone()))?;
+
+        insert_entry(txn, entry_hash, entry)?;
     }
     let dependency = get_dependency(op_light.get_type(), &header);
     let header_hashed = HeaderHashed::with_pre_hashed(header, op_light.header_hash().to_owned());
     let header_hashed = SignedHeaderHashed::with_presigned(header_hashed, signature);
     let op_order = OpOrder::new(op_light.get_type(), header_hashed.header().timestamp());
-    insert_header(txn, header_hashed)?;
-    insert_op_lite(txn, op_light, hash.clone(), op_order, timestamp)?;
+    insert_header(txn, &header_hashed)?;
+    insert_op_lite(txn, &op_light, hash, &op_order, &timestamp)?;
     set_dependency(txn, hash, dependency)?;
     Ok(())
 }
@@ -165,13 +164,13 @@ pub fn insert_op(txn: &mut Transaction, op: DhtOpHashed) -> StateMutationResult<
 /// we need the data in the same shape.
 pub fn insert_op_lite_into_authored(
     txn: &mut Transaction,
-    op_lite: DhtOpLight,
-    hash: DhtOpHash,
-    order: OpOrder,
-    timestamp: Timestamp,
+    op_lite: &DhtOpLight,
+    hash: &DhtOpHash,
+    order: &OpOrder,
+    timestamp: &Timestamp,
 ) -> StateMutationResult<()> {
-    insert_op_lite(txn, op_lite, hash.clone(), order, timestamp)?;
-    set_validation_status(txn, hash.clone(), ValidationStatus::Valid)?;
+    insert_op_lite(txn, op_lite, hash, order, timestamp)?;
+    set_validation_status(txn, hash, ValidationStatus::Valid)?;
     set_when_integrated(txn, hash, Timestamp::now())?;
     Ok(())
 }
@@ -179,10 +178,10 @@ pub fn insert_op_lite_into_authored(
 /// Insert a [`DhtOpLight`] into the database.
 pub fn insert_op_lite(
     txn: &mut Transaction,
-    op_lite: DhtOpLight,
-    hash: DhtOpHash,
-    order: OpOrder,
-    timestamp: Timestamp,
+    op_lite: &DhtOpLight,
+    hash: &DhtOpHash,
+    order: &OpOrder,
+    timestamp: &Timestamp,
 ) -> StateMutationResult<()> {
     let header_hash = op_lite.header_hash().clone();
     let basis = op_lite.dht_basis().to_owned();
@@ -194,7 +193,6 @@ pub fn insert_op_lite(
         "basis_hash": basis,
         "header_hash": header_hash,
         "require_receipt": 0,
-        "blob": to_blob(op_lite)?,
         "op_order": order,
     })?;
     Ok(())
@@ -228,8 +226,9 @@ pub fn insert_wasm(txn: &mut Transaction, wasm: DnaWasmHashed) -> StateMutationR
 }
 
 /// Insert a [`DnaDef`] into the database.
-pub fn insert_dna_def(txn: &mut Transaction, dna_def: DnaDefHashed) -> StateMutationResult<()> {
-    let (dna_def, hash) = dna_def.into_inner();
+pub fn insert_dna_def(txn: &mut Transaction, dna_def: &DnaDefHashed) -> StateMutationResult<()> {
+    let hash = dna_def.as_hash();
+    let dna_def = dna_def.as_content();
     sql_insert!(txn, DnaDef, {
         "hash": hash,
         "blob": to_blob(dna_def)?,
@@ -241,7 +240,7 @@ pub fn insert_dna_def(txn: &mut Transaction, dna_def: DnaDefHashed) -> StateMuta
 pub fn insert_entry_def(
     txn: &mut Transaction,
     key: EntryDefStoreKey,
-    entry_def: EntryDef,
+    entry_def: &EntryDef,
 ) -> StateMutationResult<()> {
     sql_insert!(txn, EntryDef, {
         "key": key,
@@ -266,7 +265,7 @@ pub fn insert_conductor_state(
 /// Set the validation status of a [`DhtOp`] in the database.
 pub fn set_validation_status(
     txn: &mut Transaction,
-    hash: DhtOpHash,
+    hash: &DhtOpHash,
     status: ValidationStatus,
 ) -> StateMutationResult<()> {
     dht_op_update!(txn, hash, {
@@ -277,7 +276,7 @@ pub fn set_validation_status(
 /// Set the integration dependency of a [`DhtOp`] in the database.
 pub fn set_dependency(
     txn: &mut Transaction,
-    hash: DhtOpHash,
+    hash: &DhtOpHash,
     dependency: Dependency,
 ) -> StateMutationResult<()> {
     match dependency {
@@ -299,7 +298,7 @@ pub fn set_dependency(
 /// Set the whether or not a receipt is required of a [`DhtOp`] in the database.
 pub fn set_require_receipt(
     txn: &mut Transaction,
-    hash: DhtOpHash,
+    hash: &DhtOpHash,
     require_receipt: bool,
 ) -> StateMutationResult<()> {
     dht_op_update!(txn, hash, {
@@ -311,7 +310,7 @@ pub fn set_require_receipt(
 /// Set the validation stage of a [`DhtOp`] in the database.
 pub fn set_validation_stage(
     txn: &mut Transaction,
-    hash: DhtOpHash,
+    hash: &DhtOpHash,
     status: ValidationLimboStatus,
 ) -> StateMutationResult<()> {
     let stage = match status {
@@ -344,7 +343,7 @@ pub fn set_validation_stage(
 /// Set when a [`DhtOp`] was integrated.
 pub fn set_when_integrated(
     txn: &mut Transaction,
-    hash: DhtOpHash,
+    hash: &DhtOpHash,
     time: Timestamp,
 ) -> StateMutationResult<()> {
     dht_op_update!(txn, hash, {
@@ -356,7 +355,7 @@ pub fn set_when_integrated(
 /// Set when a [`DhtOp`] was last publish time
 pub fn set_last_publish_time(
     txn: &mut Transaction,
-    hash: DhtOpHash,
+    hash: &DhtOpHash,
     unix_epoch: std::time::Duration,
 ) -> StateMutationResult<()> {
     dht_op_update!(txn, hash, {
@@ -366,7 +365,7 @@ pub fn set_last_publish_time(
 }
 
 /// Set withhold publish for a [`DhtOp`].
-pub fn set_withhold_publish(txn: &mut Transaction, hash: DhtOpHash) -> StateMutationResult<()> {
+pub fn set_withhold_publish(txn: &mut Transaction, hash: &DhtOpHash) -> StateMutationResult<()> {
     dht_op_update!(txn, hash, {
         "withhold_publish": true,
     })?;
@@ -374,7 +373,7 @@ pub fn set_withhold_publish(txn: &mut Transaction, hash: DhtOpHash) -> StateMuta
 }
 
 /// Unset withhold publish for a [`DhtOp`].
-pub fn unset_withhold_publish(txn: &mut Transaction, hash: DhtOpHash) -> StateMutationResult<()> {
+pub fn unset_withhold_publish(txn: &mut Transaction, hash: &DhtOpHash) -> StateMutationResult<()> {
     dht_op_update!(txn, hash, {
         "withhold_publish": Null,
     })?;
@@ -400,9 +399,16 @@ pub fn set_receipts_complete(
 }
 
 /// Insert a [`Header`] into the database.
-pub fn insert_header(txn: &mut Transaction, header: SignedHeaderHashed) -> StateMutationResult<()> {
-    let (header, signature) = header.into_header_and_signature();
-    let (header, hash) = header.into_inner();
+pub fn insert_header(
+    txn: &mut Transaction,
+    header: &SignedHeaderHashed,
+) -> StateMutationResult<()> {
+    #[derive(Serialize, Debug)]
+    struct SignedHeaderRef<'a>(&'a Header, &'a Signature);
+    let hash = header.as_hash();
+    let signature = header.signature();
+    let header = header.header();
+    let signed_header = SignedHeaderRef(header, signature);
     let header_type = header.header_type();
     let header_seq = header.header_seq();
     let author = header.author().clone();
@@ -423,7 +429,7 @@ pub fn insert_header(txn: &mut Transaction, header: SignedHeaderHashed) -> State
                 "base_hash": create_link.base_address,
                 "zome_id": create_link.zome_id.index() as u32,
                 "tag": create_link.tag,
-                "blob": to_blob(SignedHeader::from((Header::CreateLink(create_link.clone()), signature)))?,
+                "blob": to_blob(&signed_header)?,
             })?;
         }
         Header::DeleteLink(delete_link) => {
@@ -434,7 +440,7 @@ pub fn insert_header(txn: &mut Transaction, header: SignedHeaderHashed) -> State
                 "author": author,
                 "prev_hash": prev_hash,
                 "create_link_hash": delete_link.link_add_address,
-                "blob": to_blob(SignedHeader::from((Header::DeleteLink(delete_link.clone()), signature)))?,
+                "blob": to_blob(&signed_header)?,
             })?;
         }
         Header::Create(create) => {
@@ -447,7 +453,7 @@ pub fn insert_header(txn: &mut Transaction, header: SignedHeaderHashed) -> State
                 "entry_hash": create.entry_hash,
                 "entry_type": create.entry_type,
                 "private_entry": private,
-                "blob": to_blob(SignedHeader::from((Header::Create(create.clone()), signature)))?,
+                "blob": to_blob(&signed_header)?,
             })?;
         }
         Header::Delete(delete) => {
@@ -459,7 +465,7 @@ pub fn insert_header(txn: &mut Transaction, header: SignedHeaderHashed) -> State
                 "prev_hash": prev_hash,
                 "deletes_entry_hash": delete.deletes_entry_address,
                 "deletes_header_hash": delete.deletes_address,
-                "blob": to_blob(SignedHeader::from((Header::Delete(delete.clone()), signature)))?,
+                "blob": to_blob(&signed_header)?,
             })?;
         }
         Header::Update(update) => {
@@ -474,57 +480,21 @@ pub fn insert_header(txn: &mut Transaction, header: SignedHeaderHashed) -> State
                 "original_entry_hash": update.original_entry_address,
                 "original_header_hash": update.original_header_address,
                 "private_entry": private,
-                "blob": to_blob(SignedHeader::from((Header::Update(update.clone()), signature)))?,
+                "blob": to_blob(&signed_header)?,
             })?;
         }
-        Header::InitZomesComplete(izc) => {
+        Header::InitZomesComplete(_)
+        | Header::Dna(_)
+        | Header::AgentValidationPkg(_)
+        | Header::OpenChain(_)
+        | Header::CloseChain(_) => {
             sql_insert!(txn, Header, {
                 "hash": hash,
                 "type": header_type ,
                 "seq": header_seq,
                 "author": author,
                 "prev_hash": prev_hash,
-                "blob": to_blob(SignedHeader::from((Header::InitZomesComplete(izc), signature)))?,
-            })?;
-        }
-        Header::Dna(dna) => {
-            sql_insert!(txn, Header, {
-                "hash": hash,
-                "type": header_type ,
-                "seq": header_seq,
-                "author": author,
-                "prev_hash": prev_hash,
-                "blob": to_blob(SignedHeader::from((Header::Dna(dna), signature)))?,
-            })?;
-        }
-        Header::AgentValidationPkg(avp) => {
-            sql_insert!(txn, Header, {
-                "hash": hash,
-                "type": header_type ,
-                "seq": header_seq,
-                "author": author,
-                "prev_hash": prev_hash,
-                "blob": to_blob(SignedHeader::from((Header::AgentValidationPkg(avp), signature)))?,
-            })?;
-        }
-        Header::OpenChain(open) => {
-            sql_insert!(txn, Header, {
-                "hash": hash,
-                "type": header_type ,
-                "seq": header_seq,
-                "author": author,
-                "prev_hash": prev_hash,
-                "blob": to_blob(SignedHeader::from((Header::OpenChain(open), signature)))?,
-            })?;
-        }
-        Header::CloseChain(close) => {
-            sql_insert!(txn, Header, {
-                "hash": hash,
-                "type": header_type ,
-                "seq": header_seq,
-                "author": author,
-                "prev_hash": prev_hash,
-                "blob": to_blob(SignedHeader::from((Header::CloseChain(close), signature)))?,
+                "blob": to_blob(&signed_header)?,
             })?;
         }
     }
@@ -532,8 +502,11 @@ pub fn insert_header(txn: &mut Transaction, header: SignedHeaderHashed) -> State
 }
 
 /// Insert an [`Entry`] into the database.
-pub fn insert_entry(txn: &mut Transaction, entry: EntryHashed) -> StateMutationResult<()> {
-    let (entry, hash) = entry.into_inner();
+pub fn insert_entry(
+    txn: &mut Transaction,
+    hash: &EntryHash,
+    entry: &Entry,
+) -> StateMutationResult<()> {
     let mut cap_secret = None;
     let mut cap_access = None;
     let mut cap_grantor = None;
@@ -732,7 +705,7 @@ pub fn schedule_fn(
             holochain_sqlite::sql::sql_cell::schedule::UPDATE,
             named_params! {
                 ":zome_name": scheduled_fn.zome_name().to_string(),
-                ":maybe_schedule": to_blob::<Option<Schedule>>(maybe_schedule)?,
+                ":maybe_schedule": to_blob::<Option<Schedule>>(&maybe_schedule)?,
                 ":scheduled_fn": scheduled_fn.fn_name().to_string(),
                 ":start": start,
                 ":end": end,
@@ -743,7 +716,7 @@ pub fn schedule_fn(
     } else {
         sql_insert!(txn, ScheduledFunctions, {
             "zome_name": scheduled_fn.zome_name().to_string(),
-            "maybe_schedule": to_blob::<Option<Schedule>>(maybe_schedule)?,
+            "maybe_schedule": to_blob::<Option<Schedule>>(&maybe_schedule)?,
             "scheduled_fn": scheduled_fn.fn_name().to_string(),
             "start": start,
             "end": end,
